package pwc.taxtech.atms.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.ErrorMessageCN;
import pwc.taxtech.atms.dao.BusinessUnitMapper;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.entity.BusinessUnit;
import pwc.taxtech.atms.entity.BusinessUnitExample;
import pwc.taxtech.atms.entity.BusinessUnitExample.Criteria;
import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.exception.ServiceException;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 */
@Service
public class BusinessUnitServiceImpl {

    private static final Logger logger = LoggerFactory.getLogger(BusinessUnitServiceImpl.class);

    @Autowired
    private BusinessUnitMapper businessUnitMapper;

    @Autowired
    private OperationLogServiceImpl operationLogService;

    @Autowired
    private OrganizationServiceImpl organizationService;

    @Autowired
    private AuthUserHelper authUserHelper;

    public List<BusinessUnitDto> getBusinessUnits() {
        logger.debug("BusinessUnitServiceImpl getBusinessUnits");
        BusinessUnitExample businessUnitExample = new BusinessUnitExample();
        businessUnitExample.setOrderByClause("name ASC");
        List<BusinessUnit> businessUnitList = businessUnitMapper.selectByExample(businessUnitExample);
        return rotateBusinessUnitList(businessUnitList);
    }

    @Transactional
    public ApiResultDto addBusinessUnits(List<BusinessUnitInputDto> businessUnitDtoList) {
        logger.debug("BusinessUnitServiceImpl addBusinessUnits");
        ApiResultDto res = ApiResultDto.success();
        if (businessUnitDtoList.isEmpty()) {
            throw new ApplicationException(CommonConstants.JSONNULLOBJECT);
        }
        for (BusinessUnitInputDto businessUnitDto : businessUnitDtoList) {
            res = addBusinessUnit(businessUnitDto);
            if(ApiResultDto.FAILED == res.getCode()) break;
        }
        return res;
    }

    public ApiResultDto addBusinessUnit(BusinessUnitInputDto businessUnitDto) {
        BusinessUnitExample example = new BusinessUnitExample();
        example.createCriteria().andNameEqualTo(businessUnitDto.getName());
        if(businessUnitMapper.countByExample(example)>0){
            return ApiResultDto.fail(ErrorMessageCN.BusinssUnitRepeat);
        }
        BusinessUnit businessUnit = rotateBusinessUnitDtoForInput(businessUnitDto);
        businessUnit.setId(CommonUtils.getUUID());
        businessUnit.setCreateTime(new Date());
        businessUnit.setUpdateTime(new Date());
        businessUnitMapper.insert(businessUnit);

        // Log operation
        OperationLogDto operationLogDto = new OperationLogDto();
        operationLogDto.setOperationObject(businessUnitDto.getName());
        operationLogDto.setModule(OperationModule.BasicDataBusinessUnit.value());
        operationLogDto.setAction(OperationAction.New.value());
        operationLogDto.setLogType(OperateLogType.OperationLogBasicData.value());
        operationLogService.addOperationLog(operationLogDto);
        return ApiResultDto.success();
    }

    @Transactional
    public Integer deleteBusinessUnit(IdModel idModel) {
        logger.debug("BusinessUnitServiceImpl deleteBusinessUnit");

        if (idModel == null || !StringUtils.hasText(idModel.getId())) {
            throw new ApplicationException(CommonConstants.JSONNULLOBJECT);
        }

        BusinessUnit businessUnit = businessUnitMapper.selectByPrimaryKey(idModel.getId());

        if (businessUnit == null) {
            throw new ApplicationException("can't find business unit to delete, id: " + idModel.getId());
        }
        int result = businessUnitMapper.deleteByPrimaryKey(idModel.getId());

        // Log operation
        OperationLogDto operationLogDto = new OperationLogDto();
        operationLogDto.setModule(OperationModule.BasicDataBusinessUnit.value());
        operationLogDto.setAction(OperationAction.Delete.value());
        operationLogDto.setLogType(OperateLogType.OperationLogBasicData.value());
        operationLogDto.setOperationObject("");
        operationLogService.addOperationLog(operationLogDto);

        return result;
    }

    @Transactional
    public ApiResultDto updateBusinessUnits(List<BusinessUnitDto> businessUnitDtoList) {
        logger.debug("BusinessUnitServiceImpl updateBusinessUnits");

        if (businessUnitDtoList.isEmpty()) {
            return ApiResultDto.fail(CommonConstants.JSONNULLOBJECT);
        }

        for (BusinessUnitDto businessUnitDto : businessUnitDtoList) {
            // isActive changed or not
            boolean isStatusChangeOperation = false;

            if (businessUnitDto == null || !StringUtils.hasText(businessUnitDto.getId())) {
                return ApiResultDto.fail("primary key can't be null");
            }

            // current businessUnit
            BusinessUnit businessUnit = businessUnitMapper.selectByPrimaryKey(businessUnitDto.getId());
            if (businessUnit == null) {
                return ApiResultDto.fail("can't find business unit, id: " + businessUnitDto.getId());
            }
            if(businessUnitDto.getIsActive().equals(businessUnit.getIsActive())&&
                    businessUnit.getName().equals(businessUnitDto.getName())){
                return ApiResultDto.fail(ErrorMessageCN.BusinssUnitUpdateFailed);
            }

            // copy current businessUnit as tmp
            BusinessUnit originBusinessUnit = new BusinessUnit();
            CommonUtils.copyProperties(businessUnit, originBusinessUnit);

            if (businessUnitDto.getIsActive() != businessUnit.getIsActive()
                    || businessUnitDto.getName() != businessUnit.getName()) {
                isStatusChangeOperation = true;
                businessUnit.setIsActive(businessUnitDto.getIsActive());
                businessUnit.setName(businessUnitDto.getName());
                if (!businessUnitDto.getIsActive()
                        && organizationService.isOrganizationStructureExists(businessUnitDto.getId())) {
                    continue;
                }
            }
            businessUnit.setUpdateTime(new Date());
            businessUnitMapper.updateByPrimaryKey(businessUnit);

            if (isStatusChangeOperation) {
                UpdateLogParams updateLogParams = new UpdateLogParams();
                updateLogParams.setOperationObject(originBusinessUnit.getName());
                updateLogParams.setOriginalState(originBusinessUnit);
                updateLogParams.setUpdateState(businessUnit);
                updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor().get());
                updateLogParams.setOperationModule(OperationModule.BasicDataBusinessUnit.value());
                updateLogParams.setOperationAction(OperationAction.Update.value());
                updateLogParams.setOperateLogType(OperateLogType.OperationLogBasicData.value());
                // check how many properties being updated and the log number should be same
                operationLogService.updateDataAddLog(updateLogParams);
            }
        }
        return ApiResultDto.success();
    }

    private List<BusinessUnitDto> rotateBusinessUnitList(List<BusinessUnit> businessUnitList) {
        List<BusinessUnitDto> businessUnitDtoList = new ArrayList<>();
        if (businessUnitList != null && !businessUnitList.isEmpty()) {
            for (BusinessUnit businessUnit : businessUnitList) {
                businessUnitDtoList.add(rotateBusinessUnit(businessUnit));
            }
        }
        return businessUnitDtoList;
    }

    private BusinessUnitDto rotateBusinessUnit(BusinessUnit businessUnit) {
        BusinessUnitDto businessUnitDto = new BusinessUnitDto();
        businessUnitDto.setId(businessUnit.getId());
        businessUnitDto.setIsActive(businessUnit.getIsActive());
        businessUnitDto.setName(businessUnit.getName());
        return businessUnitDto;
    }

    private BusinessUnit rotateBusinessUnitDto(BusinessUnitDto businessUnitDto) {
        BusinessUnit businessUnit = new BusinessUnit();
        businessUnit.setId(businessUnitDto.getId());
        businessUnit.setName(businessUnitDto.getName());
        businessUnit.setIsActive(businessUnitDto.getIsActive());
        return businessUnit;
    }

    private BusinessUnit rotateBusinessUnitDtoForInput(BusinessUnitInputDto businessUnitDto) {
        BusinessUnit businessUnit = new BusinessUnit();
        businessUnit.setId(businessUnitDto.getId());
        businessUnit.setName(businessUnitDto.getName());
        businessUnit.setIsActive(businessUnitDto.getIsActive());
        return businessUnit;
    }

    public List<BusinessUnit> findByNameAndIsActive(String name, Boolean isActive) {
        logger.debug("This is a method similar to JPA which find business units by name and isActive");
        BusinessUnitExample example = new BusinessUnitExample();
        Criteria criteria = example.createCriteria();
        criteria.andNameEqualTo(name);
        criteria.andIsActiveEqualTo(isActive);

        return businessUnitMapper.selectByExample(example);
    }

    public List<BusinessUnit> findByIsActiveTrue() {
        BusinessUnitExample example = new BusinessUnitExample();
        example.createCriteria().andIsActiveEqualTo(true);
        return businessUnitMapper.selectByExample(example);
    }

}
