package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.DataSourceName;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.invoice.HlOutputArSjhcMapper;
import pwc.taxtech.atms.invoice.HlOutputInterfaceLineMapper;
import pwc.taxtech.atms.invoice.HlOutputOrderKpsqmxbMapper;
import pwc.taxtech.atms.invoice.HlOutputOrderKpsqzbMapper;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

public class WPSR extends FunctionBase implements FreeRefFunction {
    private HlOutputInterfaceLineMapper interfaceLineMapper;
    private HlOutputOrderKpsqmxbMapper kpsqmxbMapper;
    private HlOutputOrderKpsqzbMapper zbMapper;
    private HlOutputArSjhcMapper arSjhcMapper;

    /**
     * 无票收入
     * 参数：
     * type: 1 未开票收入  2 跨月开票
     * rate: 税率 eg: 10   16
     * valType: 1 不含税金额 2 税额
     * <p>
     * ps: 无税 =  含税/（1+税率）
     */
    public WPSR(FormulaContext formulaContext) {
        super(formulaContext);
        this.interfaceLineMapper = SpringContextUtil.getBean(HlOutputInterfaceLineMapper.class);
        this.kpsqmxbMapper = SpringContextUtil.getBean(HlOutputOrderKpsqmxbMapper.class);
        this.zbMapper = SpringContextUtil.getBean(HlOutputOrderKpsqzbMapper.class);
        this.arSjhcMapper = SpringContextUtil.getBean(HlOutputArSjhcMapper.class);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length != 3) {
            return NumberEval.ZERO;
        }
        Integer type = getIntParam(args[0], ec);
        Integer rate = getIntParam(args[1], ec);
        Integer valType = getIntParam(args[2], ec);
        String rateStr = String.valueOf(rate.doubleValue() / 100);
        if (null == type || null == rate) {
            return NumberEval.ZERO;
        }
        ValueEval result = NumberEval.ZERO;
        String formulaExpression = "WPSR(" + type + "," + rate + "," + valType + ")";
        try {
            int period = formulaContext.getPeriod();
            int year = formulaContext.getYear();
            Project project = SpringContextUtil.projectMapper.selectByPrimaryKey(formulaContext.getProjectId());
            Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(project.getOrganizationId());
            String periodStart = DateUtils.getPeriodBeginFormat(project.getYear(), period);
            String periodEnd = DateUtils.getPeriodEndFormat(project.getYear(), period);

            if (1 == type) {
                // 机构 时间范围下已确认收入的订单
                HlOutputArSjhcExample arSjhcExample = new HlOutputArSjhcExample();
                arSjhcExample.createCriteria().andHcrqBetween(periodStart, periodEnd).andStidEqualTo(organization.getCode());
                List<String> groupIdList = arSjhcMapper.selectByExample(arSjhcExample).stream().map(HlOutputArSjhc::getGroupid)
                        .collect(Collectors.toList());
                BigDecimal tmp = BigDecimal.ZERO;
                //未开票收入  全部以16%计算
                if (rate == 16) {
                    HlOutputInterfaceLineExample lineExample = new HlOutputInterfaceLineExample();
                    lineExample.createCriteria().andGroupidIn(groupIdList).andSfkpNotEqualTo("1").andContextNotEqualTo("ETMS")
                            .andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode()); //排除 状态1：已开票，即未开票和预开票
                    List<HlOutputInterfaceLine> lineList = interfaceLineMapper.selectByExample(lineExample);
                    if (!CollectionUtils.isEmpty(lineList)) {
                        tmp = tmp.add(getSumLine(lineList, NumberUtils.createDouble(rateStr), valType));
                    }
                }
                //无订单业务
                HlOutputInterfaceLineExample lineExampleWdd = new HlOutputInterfaceLineExample();
                lineExampleWdd.createCriteria().andGroupidIn(groupIdList).andContextEqualTo("ETMS")
                        .andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode()).andTaxrateEqualTo(rateStr);
                HlOutputInterfaceLineExample.Criteria criteria2 = lineExampleWdd.createCriteria().andGroupidIn(groupIdList)
                        .andContextEqualTo("ETMS").andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode())
                        .andTaxrateEqualTo(rateStr.substring(1));// 兼容税率为.16 这种数据
                lineExampleWdd.or(criteria2);
                List<HlOutputInterfaceLine> lineListWdd = interfaceLineMapper.selectByExample(lineExampleWdd);
                if (!CollectionUtils.isEmpty(lineListWdd)) {
                    tmp = tmp.add(getSumLine(lineListWdd, NumberUtils.createDouble(rateStr), valType));
                }
                result = new NumberEval(tmp.doubleValue());
            } else if (2 == type) {
                //跨月开票
                HlOutputInterfaceLineExample lineExample = new HlOutputInterfaceLineExample();
                lineExample.createCriteria().andKprqBetween(periodStart, periodEnd).andSjfyrqNotBetween(periodStart, periodEnd)
                        .andSfkpEqualTo("1").andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode());
                List<HlOutputInterfaceLine> lineList = interfaceLineMapper.selectByExample(lineExample);
                if (!CollectionUtils.isEmpty(lineList)) {
                    result = new NumberEval(getSum(lineList, rateStr, valType).doubleValue());
                }
            }
        } catch (Exception e) {
            logger.error("evaluate WPSR error.", e);
        }
        //todo data source 显示优化
        try {
            BigDecimal val = NumberUtils.createBigDecimal(((NumberEval) result).getStringValue());
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(val);
            dto.setName(DataSourceName.ReportDataSource);
            dto.setPeriod(formulaContext.getPeriod());
            dto.setProjectId(formulaContext.getProjectId());
//        dto.setRowIndex(pCell.rowIndex-1);
//        dto.setColumnIndex(pCell.columnIndex-1);
            dto.setColumnName("");
            dto.setRowName("");
            dto.setReportName(ec.getWorkbook().getSheetName(ec.getSheetIndex()));
            dto.setType(FormulaDataSourceType.WPSRSource.getCode());
            Long dataSourceId = saveDataSource(ec, Lists.newArrayList(dto), FormulaDataSourceDetailType.InputInvoiceDataSourceDto, val, formulaContext.getPeriod(),
                    formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, val, dataSourceId,
                    formulaContext.getProjectId());
        } catch (Exception e) {
            logger.error("saveDataSource or saveFormulaBlock error.", e);
        }
        return result;
    }

    /**
     * 未开票收入 接口行数据计算
     *
     * @param lineList List<HlOutputInterfaceLine>
     * @param rate     eg：0.16
     * @param valType  1 不含税金额 2 税额
     * @return BigDecimal
     * <p>
     * ps: 无税 =  含税/（1+税率）
     */
    private BigDecimal getSumLine(List<HlOutputInterfaceLine> lineList, Double rate, Integer valType) {
        if (CollectionUtils.isEmpty(lineList) || (1 != valType && 2 != valType)) {
            return BigDecimal.ZERO;
        }
        if (1 == valType) {
            return lineList.stream().map(o -> NumberUtils.createBigDecimal(o.getSrje())
                    .divide(new BigDecimal(((Double) (rate + 1)).toString()), 2, BigDecimal.ROUND_HALF_UP))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
        } else {
            return lineList.stream().map(o -> {
                BigDecimal total = NumberUtils.createBigDecimal(o.getSrje());
                BigDecimal amount = total.divide(new BigDecimal(((Double) (rate + 1)).toString()), 2, BigDecimal.ROUND_HALF_UP);
                return total.subtract(amount);
            }).reduce(BigDecimal.ZERO, BigDecimal::add);
        }
    }

    /**
     * 接口行数据 -> 发票明细  求不含税金额总和
     *
     * @param lineList List<HlOutputInterfaceLine>
     * @param rateStr  eg：0.16
     * @param valType  1 不含税金额 2 税额
     * @return BigDecimal
     */
    private BigDecimal getSum(List<HlOutputInterfaceLine> lineList, String rateStr, Integer valType) {
        if (1 != valType && 2 != valType) {
            return BigDecimal.ZERO;
        }
        List<String> idList = lineList.stream().map(HlOutputInterfaceLine::getUuid).collect(Collectors.toList());
        HlOutputOrderKpsqmxbExample mxbExample = new HlOutputOrderKpsqmxbExample();
        mxbExample.createCriteria().andIdIn(idList);
        List<String> sqList = kpsqmxbMapper.selectByExample(mxbExample).stream().map(HlOutputOrderKpsqmxb::getSquuid)
                .collect(Collectors.toList());
        HlOutputOrderKpsqzbExample zbExample = new HlOutputOrderKpsqzbExample();
        zbExample.createCriteria().andSquuidIn(sqList);
        List<String> sqbhList = zbMapper.selectByExample(zbExample).stream().map(HlOutputOrderKpsqzb::getSqbh)
                .collect(Collectors.toList());
        OutputInvoiceExample outputInvoiceExample = new OutputInvoiceExample();
        outputInvoiceExample.createCriteria().andSQBHIn(sqbhList);
        List<String> lshList = SpringContextUtil.outputInvoiceMapper.selectByExample(outputInvoiceExample).stream()
                .map(OutputInvoice::getFPQQLSH).collect(Collectors.toList());
        OutputInvoiceDetailExample detailExample = new OutputInvoiceDetailExample();
        detailExample.createCriteria().andFPQQLSHIn(lshList).andSLVEqualTo(rateStr);
        if (1 == valType) {
            return SpringContextUtil.outputInvoiceDetailMapper.selectByExample(detailExample).stream()
                    .map(o -> NumberUtils.createBigDecimal(o.getJE())).reduce(BigDecimal.ZERO, BigDecimal::add);
        } else {
            return SpringContextUtil.outputInvoiceDetailMapper.selectByExample(detailExample).stream()
                    .map(o -> NumberUtils.createBigDecimal(o.getSE())).reduce(BigDecimal.ZERO, BigDecimal::add);
        }
    }
}
